@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* gtkhex.c - a GtkHex widget, modified for use in GHex
 
-   Copyright (C) 1998 - 2002 Free Software Foundation
+   Copyright (C) 1998 - 2003 Free Software Foundation
 
    GHex is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -43,9 +43,6 @@
 #  define is_displayable(c) (((c) >= 0x20) && ((c) <= 0x7f))
 #endif /* 0 */
 
-static GdkColor cl_cursor_a = { 0, 0xFFFF, 0, 0 };
-static GdkColor cl_cursor_p = { 0, 0xFFFF, 0x7F00, 0x7F00 };
-
 typedef void (*DataChangedSignal)(GtkObject *, gpointer, gpointer);
 
 enum {
@@ -320,7 +317,6 @@ static void render_byte(GtkHex *gh, gint pos) {
 		/* Changes for Gnome 2.0 */
 		pango_layout_set_text (gh->xlayout, buf, 2);
 		gdk_draw_layout (gh->xdisp->window, gh->xdisp_gc, cx, cy, gh->xlayout);
-
 	}
 	
 	if(!get_acoords(gh, pos, &cx, &cy))
@@ -337,7 +333,6 @@ static void render_byte(GtkHex *gh, gint pos) {
 		/* Changes for Gnome 2.0 */
 		pango_layout_set_text (gh->alayout, buf, 1);
 		gdk_draw_layout (gh->adisp->window, gh->adisp_gc, cx, cy, gh->alayout);
-
 	}
 }
 
@@ -358,10 +353,10 @@ static void render_ac(GtkHex *gh) {
 			c[0] = '.';
 
 		if(gh->active_view == VIEW_ASCII) {
-			gdk_gc_set_foreground(gh->adisp_gc, &klass->cl_cursor_a);
+			gdk_gc_set_foreground(gh->adisp_gc, &GTK_WIDGET(gh)->style->base[GTK_STATE_ACTIVE]);
 		}
 		else {
-			gdk_gc_set_foreground(gh->adisp_gc, &klass->cl_cursor_p);
+			gdk_gc_set_foreground(gh->adisp_gc, &GTK_WIDGET(gh)->style->fg[GTK_STATE_INSENSITIVE]);
 		}
 
 		gdk_draw_rectangle(gh->adisp->window, gh->adisp_gc,
@@ -393,10 +388,10 @@ static void render_xc(GtkHex *gh) {
 		}
 
 		if(gh->active_view == VIEW_HEX) {
-			gdk_gc_set_foreground(gh->xdisp_gc, &klass->cl_cursor_a);
+			gdk_gc_set_foreground(gh->xdisp_gc, &GTK_WIDGET(gh)->style->base[GTK_STATE_ACTIVE]);
 		}
 		else {
-			gdk_gc_set_foreground(gh->xdisp_gc, &klass->cl_cursor_p);
+			gdk_gc_set_foreground(gh->xdisp_gc, &GTK_WIDGET(gh)->style->fg[GTK_STATE_INSENSITIVE]);
 		}
 
 		gdk_draw_rectangle(GTK_WIDGET(gh->xdisp)->window, gh->xdisp_gc,
@@ -437,6 +432,7 @@ static void render_hex_lines(GtkHex *gh, gint imin, gint imax) {
 	gint i, cursor_line, sl, el, start, end;
 	gint xcpl = gh->cpl*2 + gh->cpl/gh->group_type;
 	gint frm_len, tmp, cursor_off = 0, len;
+	GtkStateType state;
 
 	if( (!GTK_WIDGET_REALIZED(gh)) || (gh->cpl == 0) )
 		return;
@@ -474,6 +470,7 @@ static void render_hex_lines(GtkHex *gh, gint imin, gint imax) {
 			return;
 
 		if(gh->sel_start != gh->sel_end) {
+			state = (gh->active_view == VIEW_HEX)?GTK_STATE_ACTIVE:GTK_STATE_INSENSITIVE;
 			if(i == sl) {
 				cursor_off = 2*(start%gh->cpl) + (start%gh->cpl)/gh->group_type;
 				if(i == el)
@@ -483,7 +480,7 @@ static void render_hex_lines(GtkHex *gh, gint imin, gint imax) {
 				len = len - cursor_off;
 				if(len > 0)
 					gtk_draw_box(GTK_WIDGET(gh)->style, gh->xdisp->window,
-								 GTK_STATE_SELECTED, GTK_SHADOW_NONE,
+								 state, GTK_SHADOW_NONE,
 								 cursor_off*gh->char_width, i*gh->char_height,
 								 len*gh->char_width, gh->char_height);
 			}
@@ -491,13 +488,13 @@ static void render_hex_lines(GtkHex *gh, gint imin, gint imax) {
 				cursor_off = 2*(end%gh->cpl + 1) + (end%gh->cpl)/gh->group_type;
 				if(cursor_off > 0)
 					gtk_draw_box(GTK_WIDGET(gh)->style, gh->xdisp->window,
-								 GTK_STATE_SELECTED, GTK_SHADOW_NONE,
+								 state, GTK_SHADOW_NONE,
 								 0, i*gh->char_height,
 								 cursor_off*gh->char_width, gh->char_height);
 			}
 			else if(i > sl && i < el) {
 				gtk_draw_box(GTK_WIDGET(gh)->style, gh->xdisp->window,
-							 GTK_STATE_SELECTED, GTK_SHADOW_NONE,
+							 state, GTK_SHADOW_NONE,
 							 0, i*gh->char_height,
 							 xcpl*gh->char_width, gh->char_height);
 			}
@@ -515,7 +512,8 @@ static void render_ascii_lines(GtkHex *gh, gint imin, gint imax) {
 	GtkWidget *w = gh->adisp;
 	gint i, tmp, frm_len, sl, el, start, end;
 	guint cursor_line, cursor_off, len;
-	
+	GtkStateType state;
+
 	if( (!GTK_WIDGET_REALIZED(gh)) || (gh->cpl == 0) )
 		return;
 	
@@ -552,6 +550,7 @@ static void render_ascii_lines(GtkHex *gh, gint imin, gint imax) {
 			return;
 
 		if(gh->sel_start != gh->sel_end) {
+			state = (gh->active_view == VIEW_HEX)?GTK_STATE_ACTIVE:GTK_STATE_INSENSITIVE;
 			if(i == sl) {
 				cursor_off = start%gh->cpl;
 				if(i == el)
@@ -560,7 +559,7 @@ static void render_ascii_lines(GtkHex *gh, gint imin, gint imax) {
 					len = gh->cpl - cursor_off;
 				if(len > 0)
 					gtk_draw_box(GTK_WIDGET(gh)->style, gh->adisp->window,
-								 GTK_STATE_SELECTED, GTK_SHADOW_NONE,
+								 state, GTK_SHADOW_NONE,
 								 cursor_off*gh->char_width, i*gh->char_height,
 								 len*gh->char_width, gh->char_height);
 			}
@@ -568,13 +567,13 @@ static void render_ascii_lines(GtkHex *gh, gint imin, gint imax) {
 				cursor_off = end%gh->cpl + 1;
 				if(cursor_off > 0)
 					gtk_draw_box(GTK_WIDGET(gh)->style, gh->adisp->window,
-								 GTK_STATE_SELECTED, GTK_SHADOW_NONE,
+								 state, GTK_SHADOW_NONE,
 								 0, i*gh->char_height,
 								 cursor_off*gh->char_width, gh->char_height);
 			}
 			else if(i > sl && i < el) {
 				gtk_draw_box(GTK_WIDGET(gh)->style, gh->adisp->window,
-							 GTK_STATE_SELECTED, GTK_SHADOW_NONE,
+							 state, GTK_SHADOW_NONE,
 							 0, i*gh->char_height,
 							 gh->cpl*gh->char_width, gh->char_height);
 			}
@@ -872,7 +871,6 @@ static gboolean scroll_timeout_handler(GtkHex *gh) {
 }
 
 static void hex_button_cb(GtkWidget *w, GdkEventButton *event, GtkHex *gh) {
-	
 	if( (event->type == GDK_BUTTON_RELEASE) &&
 		(event->button == 1) ) {
 		if(gh->scroll_timeout != -1) {
@@ -906,6 +904,22 @@ static void hex_button_cb(GtkWidget *w, GdkEventButton *event, GtkHex *gh) {
 			show_cursor(gh);
 		}
 	}
+	else if((event->type == GDK_BUTTON_PRESS) && (event->button == 2)) {
+		GtkHexClass *klass = GTK_HEX_CLASS(GTK_WIDGET_GET_CLASS(gh));
+		gchar *text;
+
+		gh->active_view = VIEW_HEX;
+		hex_to_pointer(gh, event->x, event->y);
+
+		text = gtk_clipboard_wait_for_text(klass->primary);
+		if(text) {
+			hex_document_set_data(gh->document, gh->cursor_pos,
+								  strlen(text), 0, text, TRUE);
+			gtk_hex_set_cursor(gh, gh->cursor_pos + strlen(text));
+			g_free(text);
+		}
+		gh->button = 0;
+	}
 	else
 		gh->button = 0;
 }
@@ -973,6 +987,22 @@ static void ascii_button_cb(GtkWidget *w, GdkEventButton *event, GtkHex *gh) {
 			show_cursor(gh);
 		}
 	}
+	else if((event->type == GDK_BUTTON_PRESS) && (event->button == 2)) {
+		GtkHexClass *klass = GTK_HEX_CLASS(GTK_WIDGET_GET_CLASS(gh));
+		gchar *text;
+
+		gh->active_view = VIEW_ASCII;
+		ascii_to_pointer(gh, event->x, event->y);
+
+		text = gtk_clipboard_wait_for_text(klass->primary);
+		if(text) {
+			hex_document_set_data(gh->document, gh->cursor_pos,
+								  strlen(text), 0, text, TRUE);
+			gtk_hex_set_cursor(gh, gh->cursor_pos + strlen(text));
+			g_free(text);
+		}
+		gh->button = 0;
+	}
 	else
 		gh->button = 0;
 }
@@ -1126,8 +1156,6 @@ static void primary_get_cb(GtkClipboard *clipboard,
 		guchar *text;
 		GtkHexClass *klass = GTK_HEX_CLASS(GTK_WIDGET_GET_CLASS(gh));
 
-		g_message("copy_primary");
-
 		start_pos = MIN(gh->sel_start, gh->sel_end);
 		end_pos = MAX(gh->sel_start, gh->sel_end);
  
@@ -1198,8 +1226,11 @@ void gtk_hex_delete_selection(GtkHex *gh)
 
 	gtk_hex_select_region(gh, 0, 0);
 
-	if(start != end)
+	if(start != end) {
+		if(start < gh->cursor_pos)
+			gtk_hex_set_cursor(gh, gh->cursor_pos - end + start);
 		hex_document_delete_data(gh->document, MIN(start, end), end - start, TRUE);
+	}
 }
 
 void gtk_hex_copy_clipboard(GtkHex *gh)
@@ -1224,7 +1255,6 @@ static void gtk_hex_real_copy_clipboard(GtkHex *gh)
 	gint end_pos;
 	GtkHexClass *klass = GTK_HEX_CLASS(GTK_WIDGET_GET_CLASS(gh));
 
-	g_message("copy_clipboard");
 	start_pos = MIN(gh->sel_start, gh->sel_end);
 	end_pos = MAX(gh->sel_start, gh->sel_end);
  
@@ -1238,8 +1268,6 @@ static void gtk_hex_real_copy_clipboard(GtkHex *gh)
 
 static void gtk_hex_real_cut_clipboard(GtkHex *gh)
 {
-	g_message("cut_clipboard");
-
 	if(gh->sel_start != -1 && gh->sel_end != -1) {
 		gtk_hex_real_copy_clipboard(gh);
 		gtk_hex_delete_selection(gh);
@@ -1251,12 +1279,11 @@ static void gtk_hex_real_paste_clipboard(GtkHex *gh)
 	GtkHexClass *klass = GTK_HEX_CLASS(GTK_WIDGET_GET_CLASS(gh));
 	gchar *text;
 
-	g_message("paste_clipboard");
-
 	text = gtk_clipboard_wait_for_text(klass->clipboard);
 	if(text) {
 		hex_document_set_data(gh->document, gh->cursor_pos,
 							  strlen(text), 0, text, TRUE);
+		gtk_hex_set_cursor(gh, gh->cursor_pos + strlen(text));
 		g_free(text);
 	}
 }
@@ -1306,6 +1333,8 @@ static gint gtk_hex_key_press(GtkWidget *w, GdkEventKey *event) {
 		if(gh->cursor_pos > 0) {
 			hex_document_set_data(gh->document, gh->cursor_pos - 1,
 								  0, 1, NULL, TRUE);
+			if (gh->selecting)
+				gh->selecting = FALSE;
 			gtk_hex_set_cursor(gh, gh->cursor_pos - 1);
 		}
 		break;
@@ -1382,6 +1411,8 @@ static gint gtk_hex_key_press(GtkWidget *w, GdkEventKey *event) {
 					hex_document_set_nibble(gh->document, event->keyval - '0',
 											gh->cursor_pos, gh->lower_nibble,
 											gh->insert, TRUE);
+					if (gh->selecting)
+						gh->selecting = FALSE;
 					gh->lower_nibble = !gh->lower_nibble;
 					if(!gh->lower_nibble)
 						gtk_hex_set_cursor(gh, gh->cursor_pos + 1);
@@ -1390,6 +1421,8 @@ static gint gtk_hex_key_press(GtkWidget *w, GdkEventKey *event) {
 					hex_document_set_nibble(gh->document, event->keyval - 'A' + 10,
 											gh->cursor_pos, gh->lower_nibble,
 											gh->insert, TRUE);
+					if (gh->selecting)
+						gh->selecting = FALSE;
 					gh->lower_nibble = !gh->lower_nibble;
 					if(!gh->lower_nibble)
 						gtk_hex_set_cursor(gh, gh->cursor_pos + 1);
@@ -1398,6 +1431,8 @@ static gint gtk_hex_key_press(GtkWidget *w, GdkEventKey *event) {
 					hex_document_set_nibble(gh->document, event->keyval - 'a' + 10,
 											gh->cursor_pos, gh->lower_nibble,
 											gh->insert, TRUE);
+					if (gh->selecting)
+						gh->selecting = FALSE;
 					gh->lower_nibble = !gh->lower_nibble;
 					if(!gh->lower_nibble)
 						gtk_hex_set_cursor(gh, gh->cursor_pos + 1);
@@ -1423,6 +1458,8 @@ static gint gtk_hex_key_press(GtkWidget *w, GdkEventKey *event) {
 				else if(is_displayable(event->keyval)) {
 					hex_document_set_byte(gh->document, event->keyval,
 										  gh->cursor_pos, gh->insert, TRUE);
+					if (gh->selecting)
+						gh->selecting = FALSE;
 					old_cp = gh->cursor_pos;
 					gtk_hex_set_cursor(gh, gh->cursor_pos + 1);
 				}
@@ -1572,13 +1609,6 @@ static void gtk_hex_class_init(GtkHexClass *klass, gpointer data) {
 	/* Changed in Gnome 2.0 -- SnM */
 	G_OBJECT_CLASS(klass)->finalize = gtk_hex_finalize;
 
-	klass->cmap = gdk_colormap_get_system();
-	
-	memcpy(&klass->cl_cursor_p, &cl_cursor_p, sizeof(GdkColor));
-	memcpy(&klass->cl_cursor_a, &cl_cursor_a, sizeof(GdkColor));
-	gdk_colormap_alloc_color(klass->cmap, &klass->cl_cursor_p, FALSE, TRUE);
-	gdk_colormap_alloc_color(klass->cmap, &klass->cl_cursor_a, FALSE, TRUE);
-
 	parent_class = gtk_type_class (gtk_fixed_get_type ());
 }
 
